# -*-python-*-

__package__    = "s60-restobook"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/restobook/"
__cvsversion__ = "$Revision: 1.13 $"
__date__       = "$Date: 2006/07/21 22:38:08 $"
__copyright__  = "Copyright (c) 2006 Aaron Straup Cope. Perl Artistic License."

import re
import e32
import anydbm
import appuifw
import base64
import md5
import os.path
# import contacts

from elementtree.ElementTree import XML
from pdis.xpath import compile
import httplib
import urllib

class Resto :

    def __init__ (self) :

        self.__msg = ""

        if not e32.in_emulator() :
            self.__db  = contacts.open()

        self.__cfgpath = u"c:\\restobook.db"     
        self.lock  = e32.Ao_lock()

        self.__addr_list = ("street_address", "city", "state", "postal_code", "country")
        self.__common    = {"title":"company_name", "phone":"phone_number", "email":"email_address"}
        
        self.old_title = appuifw.app.title
        appuifw.app.title = u"restobook"

        self.exit_flag = False
        appuifw.app.exit_key_handler = self.abort

        appuifw.app.menu = []

    #
    #
    #

    def loop (self) :
        self.run()
        self.lock.wait()

    #
    #
    #
    
    def abort(self) :
        self.store_cfg()
        self.lock.signal()
        self.old_title = appuifw.app.title

    #
    #
    #

    def open_cfg (self) :
        self.log("Reading configs")

        if not os.path.exists(self.__cfgpath) :
            self.__cfg = anydbm.open(self.__cfgpath, "c")
            self.__cfg.close()

        self.__cfg = anydbm.open(self.__cfgpath, "w")
        return True

    #
    #
    #
    
    def store_cfg (self) :
        self.__cfg.close()
            
    #
    #
    #
    
    def run (self) :

        self.log("restobook says hello!")
        self.log("---------------------")
        
        if not self.open_cfg() :
            return False
        
        if not self.test_connection() :
            return False
        
        if not self.get_credentials() :
            return False
        
        if not self.test_credentials() :

            if appuifw.query(u"Login failed. try again?", "query") :
                self.change_credentials()
                return self.run()
            else :
                return False

        # merge-iness

        opts   = [u"Merge (default)", u"Skip", u"Ask first"]
        prompt = u"How should restobook handle existing records"
        merge  = appuifw.popup_menu(opts, prompt)

        if merge == None :
            merge = 0
            
        self.__cfg["merge"] = unicode(merge)
        
        #
        
        dates = self.fetch_dates()

        if len(dates) == 0 :
            self.error("Hrm...there was a problem rounding up restobook dates")
            return False
            
        posts = self.fetch_posts(dates)

        if posts == None or len(posts) == 0 :
            self.error("No suitable posts were found.")
            return False
        
        restos = self.parse_posts(posts)

        if e32.in_emulator() :
            self.dump_restos(restos)
        else :
            for resto in restos :
                self.store_resto(resto)

        self.log("All done! Mange! Mange!!")
        return True

    #
    #
    #
    
    def dump_restos (self, restos) :
        
        for r in restos :

            self.log("%s (dump) :" % r['company_name'])
            self.log("--------------------------------")
            
            for k, v in r.items() :
                if k == 'uri' :
                    for label, link in r[k].items() :
                        self.log("%s (%s)" % (link, label))
                else :
                    self.log("%s : %s" % (k, v))

            self.log("--------------------------------")
            
    #
    #
    #

    def change_credentials (self) :

        for prop in ("user", "pswd") :
            if self.__cfg.has_key(prop) :
                del(self.__cfg[prop])

        self.store_cfg()
        return self.get_credentials()

    #
    #
    #
    
    def get_credentials (self) :

        user = None
        pswd = None
        
        if not self.__cfg.has_key("user") :
            user = appuifw.query(u"your del. username", "text")

            if user == None :
                return False

            self.__cfg["user"] = user
        
        if not self.__cfg.has_key("pswd") :
            pswd = appuifw.query(u"your del. pswd", "text")

            if pswd == None :
                return False

            self.__cfg["pswd"] = pswd

        return True
            
    #
    #
    #

    def test_credentials (self) :
        self.log("Testing your del. login")
        
        res = self.execute_delicious_request("/v1/posts/update")

        if res == None :
            return False

        return True

    #
    #
    #
    
    def test_connection (self) :
        self.log("Testing your network connection")
        
        try :
            conn = httplib.HTTPConnection('del.icio.us')
            conn.request('GET', '/')
        except Exception, e :
            self.error("Can not find the network love (%s) :-(" % e)
            return False

        return True
    
    #
    #
    #
    
    def fetch_posts (self, dates) :
        posts = []
        
        for dt in dates :
            self.log("Fetching posts tagged with restobook on %s" % dt)

            params = urllib.urlencode({'tag':'restobook', 'dt':dt})
            req    = "/v1/posts/get?%s" % params

            xml = self.execute_delicious_request(req)

            if xml == None :
                self.error("Hrm...couldn't get posts for %s" % dt)
                continue
            
            xpath = compile("/posts/post")
            
            for p in xpath.evaluate(xml) :

                if self.add_link(p) :                    
                    posts.append(p)

        return posts
    
    #
    #
    #
    
    def fetch_dates (self) :
        self.log("Fetching dates tagged with restobook")        
        dates = []
        
        req = "/v1/posts/dates?tag=restobook"
        xml = self.execute_delicious_request(req)

        if xml == None :
            return dates
        
        xpath = compile("dates/date")
        res   = xpath.evaluate(xml)
        
        for dt in res :
            dates.append(dt.attrib['date'])

        return dates
    
    #
    #
    #
    
    def execute_delicious_request (self, req) :

        base64string = base64.encodestring('%s:%s' % (self.__cfg['user'], self.__cfg['pswd']))[:-1]
        headers = {"Authorization":"Basic %s" % base64string}
        
        url = "api.del.icio.us"
    
        conn = httplib.HTTPSConnection(url)
        conn.request('GET', req, None, headers)

        res = conn.getresponse()

        if res.status != 200 :
            self.error(res.reason)
            return None

        content = res.read()
        # print content
        
        return XML(content)

    #
    #
    #
    
    def geocode_address (self, addr, resto) :
        self.log("Trying to geocode %s" % addr)
        
        params = urllib.urlencode({"appid":"restobook", "location":addr})
        req    = "/MapsService/V1/geocode?%s" % params

        xml = self.execute_yahoo_request(req)
        res = xml.find("{urn:yahoo:maps}Result")

        if res == None :
            return False

        findme = list(self.__addr_list)
        findme.append("latitude")
        findme.append("longitude")

        for prop in findme :

            ykey = prop.title()
            
            if prop == "street_address" :
                ykey = "Address"
            elif prop == "postal_code" :
                ykey = "Zip"
            else :
                pass

            el   = "{urn:yahoo:maps}%s" % ykey
            node = res.find(el)

            if node == None:
                resto[prop.lower()] = ""                                
            else :
                resto[prop.lower()] = node.text.title()

        resto['geo_address'] = addr
        return True
        
    #
    #
    #
    
    def execute_yahoo_request(self, req) :

        url = "api.local.yahoo.com"
        
        conn = httplib.HTTPConnection(url)
        conn.request('GET', req)

        res = conn.getresponse()

        if res.status != 200 :
            self.error(res.reason)
            return None

        content = res.read()
        # print content
        
        return XML(content)

        
    #
    #
    #
    
    def parse_posts (self, posts) :
        self.log("Parsing %d results" % len(posts))

        parsed = []

        for r in posts :

            title = r.attrib["description"]
            self.log("Parsing data for %s" % title)
            
            resto = self.parse_n3card(r.attrib["extended"])

            #

            # sh : short(hand)
            # lh : longhand
            
            for sh, lh in self.__common.items() :
                if not resto.has_key(sh) :
                    continue

                if not resto.has_key(lh) :
                    resto[lh] = resto[sh]

                del(resto[sh])

            #

            link = r.attrib["href"]

            #
            # Fix me : based on an as-yet defined/collected
            # pref, check to see whether this record exists
            # and whether we even want to update it
            # 
            
            resto['company_name'] = r.attrib["description"]
            resto['tags']  = r.attrib["tag"]
            resto['uri']   = {"homepage":link}

            if resto.has_key("yelp") :
                resto['uri']['yelp'] = "http://www.yelp.com/biz/%s" % resto['yelp']
                del(resto['yelp'])
                
            if resto.has_key("latitude") and resto.has_key("longitude") :
                coords = "%s, %s" % (resto['latitude'], resto['longitude'])
                params = urllib.urlencode({'addr':resto['geo_address']})

                resto['uri']['map'] = "http://maps.yahoo.com/py/maps.py?%s" % params
                resto['geopostion'] = coords
                
                del(resto['latitude'])
                del(resto['longitude'])
                del(resto['geo_address'])                                
                
            hex = md5.md5(link).hexdigest()
            resto['uri']['del.icio.us'] = "http://del.icio.us/url/%s" % hex
            
            parsed.append(resto)

        return parsed

    #
    #
    #

    def add_link (self, post) :
        merge = int(self.__cfg["merge"])
        
        if not merge :
            return True
        
        link  = post.attrib["href"]
        title = post.attrib["description"]
        
        if not self.find_resto(link) :
            return True

        if merge == 1 :
            self.log("Skip %s" % title)
            return False

        if not appuifw.query(u"Merge %s?" % title, "query") :
            self.log("Skip %s" % title)
            return False

        return True
    
    #
    #
    #
    
    def store_resto (self, data) :

        resto = self.find_resto(data['uri']['homepage'])
        
        if resto == None :
            self.log("Adding %s" % data['company_name'])
            self.add_resto(data)
        else :
            self.log("Updating %s" % data['company_name'])   
            self.update_resto(resto, data)

    #
    #
    #
    
    def find_resto (self, uri) :

        if e32.in_emulator() :
            return None
        
        res = self.__db.find(uri)

        if len(res) == 0 :
            return None

        return res[0]

    #
    #
    #

    def add_resto (self, data) :

        resto = self.__db.add_contact()
        basic = self.basic_fields()
                
        for prop in data :
            if prop in basic :
                resto.add_field(prop, value=data[prop])
            elif prop == 'uri' :
                for link in data['uri'] :
                    resto.add_field('url', value=data['uri'][link], label=link)
            else :
                resto.add_field('note', value=data[prop], label=prop)

        resto.commit()
        
    #
    #
    #

    def basic_fields (self) :
        fixme = []
        for k in self.__addr_list :
            fixme.append(k)
        for k in self.__common.values() :
            fixme.append(k)

        return fixme

    #
    #
    #
    
    def update_resto (self, resto, data) :
        resto.begin()

        basic = self.basic_fields()
        
        for prop in data :
            if prop in basic :
                found = resto.find(type=prop)

                if len(found) :
                    found[0].value = data[prop]
                else :
                    resto.add_field(prop, value=data[prop])

            elif prop == 'uri' :
                for link in data['uri'] :

                    found = 0
                    
                    for field in resto.find(type='url') :
                        if field.label == link :
                            field.value = data['uri'][link]
                            found = 1
                            break
                        
                    if not found :
                        resto.add_field('url', value=data['uri'][link], label=link)
            else :

                found = 0

                for field in resto.find(type='note') :
                    if field.label == prop :
                        field.value = data[prop]
                        found = 1
                        break

                if not found :
                    resto.add_field('note', value=data[prop], label=prop)

        resto.commit()
            
    #
    #
    #
    
    def parse_n3card (self, n3) :

        resto = {}

        matches = re.findall(':(?:([^\s]+)\s+"([^"]+)";\s?)', n3)

        for pair in matches :
            predicate = pair[0]
            object = pair[1]

            if predicate == 'address' :
                if self.geocode_address(unicode(object), resto) :
                    continue
                
            resto[predicate] = object

        return resto

    #
    #
    #

    def error (self, msg) :
        self.log(msg, 1)

    #
    #
    #
    
    def log (self, msg, error=0) :

        if e32.in_emulator() :
            print msg
            return

        self.__msg = unicode("%s\n%s" % (self.__msg, msg))

        t = appuifw.Text(self.__msg)

        if error :
            t.color = (255, 0, 0)
        
        appuifw.app.body = t
        
    #
    #
    #
    
if __name__ == "__main__" :
    app = Resto()
    app.loop()
